@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: AssistantPanel.cpp 3380 2008-08-30 09:43:02Z ippei $
+ *  $Id: AssistantPanel.cpp 3944 2009-06-17 05:34:34Z tmodes $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -28,6 +28,8 @@
 #include "panoinc_WX.h"
 #include "panoinc.h"
 
+#include "base_wx/platform.h"
+
 #include "common/stl_utils.h"
 
 #include <map>
@@ -54,6 +56,11 @@
 #include "base_wx/MyProgressDialog.h"
 #include "hugin/config_defaults.h"
 
+// Celeste header
+#include "Celeste.h"
+#include "CelesteGlobals.h"
+#include "Utilities.h"
+
 using namespace PT;
 using namespace PTools;
 using namespace utils;
@@ -379,9 +386,9 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
     bool createCtrlP = true;
     // TODO: handle existing control points properly instead of adding them twice.
     if (m_pano->getNrOfCtrlPoints() > 0) {
-        int a = wxMessageBox(wxString::Format(_("The panorama already has %d control points.\n\nSkip control points creation?"), m_pano->getNrOfCtrlPoints()),
-                     _("Skip control point creation?"), wxICON_QUESTION | wxYES_NO);
-        createCtrlP = a != wxYES;
+    int a = wxMessageBox(wxString::Format(_("The panorama already has %d control points.\n\nSkip control points creation?"), m_pano->getNrOfCtrlPoints()),
+    _("Skip control point creation?"), wxICON_QUESTION | wxYES_NO);
+    createCtrlP = a != wxYES;
     }
     */
 
@@ -393,11 +400,131 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
         AutoCtrlPointCreator matcher;
         CPVector cps = matcher.automatch(*m_pano, imgs, nFeatures,this);
         GlobalCmdHist::getInstance().addCommand(
-                new PT::AddCtrlPointsCmd(*m_pano, cps)
-                                               );
+            new PT::AddCtrlPointsCmd(*m_pano, cps)
+            );
     }
 
-    progress.increaseProgress(1.0, std::string(wxString(_("Determining placement of the images")).mb_str(wxConvLocal)));
+    // Run Celeste
+    bool t = (wxConfigBase::Get()->Read(wxT("/Celeste/Auto"), HUGIN_CELESTE_AUTO) != 0); 
+    if (t && m_pano->getNrOfCtrlPoints())
+    {
+
+        DEBUG_TRACE("Running Celeste");
+        progress.increaseProgress(1, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+
+        // set numeric locale to C, for correct number output
+        char * old_locale = setlocale(LC_NUMERIC,NULL);
+        setlocale(LC_NUMERIC,"C");
+
+        // determine file name of SVM model file
+        // get XRC path from application
+        wxString wxstrModelFileName = huginApp::Get()->GetXRCPath() + wxT("data/") + wxT(HUGIN_CELESTE_MODEL);
+        // convert wxString to string
+        string strModelFileName(wxstrModelFileName.mb_str(wxConvUTF8));
+		
+        // SVM model file
+        if ( wxFile::Exists(wxstrModelFileName) )
+        {
+
+            for (unsigned int imgNr = 0; imgNr < m_pano->getNrOfImages() - 1; imgNr++){
+
+                double progress_amount =  (double)1/m_pano->getNrOfImages();
+                progress.increaseProgress(progress_amount, std::wstring(wxString(_("Running Celeste")).wc_str(wxConvLocal)));
+
+                const CPVector & controlPoints = m_pano->getCtrlPoints();
+                unsigned int removed = 0;
+
+                gNumLocs = 0;
+                for (PT::CPVector::const_iterator it = controlPoints.begin(); it != controlPoints.end(); ++it) {
+                    PT::ControlPoint point = *it;
+                    if (imgNr == point.image1Nr){
+                        gNumLocs++;				
+                    }
+                    if (imgNr == point.image2Nr){
+                        gNumLocs++;				
+                    }	
+                }		
+
+                // Create the storage matrix
+                gLocations = CreateMatrix( (int)0, gNumLocs, 2);
+                unsigned int glocation_counter = 0;
+                unsigned int cp_counter = 0;	
+                vector<unsigned int> global_cp_nr;
+
+                for (PT::CPVector::const_iterator it = controlPoints.begin(); it != controlPoints.end(); ++it) {
+                    PT::ControlPoint point = *it;
+                    if (imgNr == point.image1Nr){	
+                        gLocations[glocation_counter][0] = (int)point.x1;
+                        gLocations[glocation_counter][1] = (int)point.y1;
+                        global_cp_nr.push_back(cp_counter);	
+                        glocation_counter++;				
+                    }
+                    if (imgNr == point.image2Nr){
+                        gLocations[glocation_counter][0] = (int)point.x2;
+                        gLocations[glocation_counter][1] = (int)point.y2;
+                        global_cp_nr.push_back(cp_counter);	
+                        glocation_counter++;				
+                    }
+                    cp_counter++;	
+                }
+
+                // SVM threshold
+                double threshold = HUGIN_CELESTE_THRESHOLD;
+                wxConfigBase::Get()->Read(wxT("/Celeste/Threshold"), &threshold, HUGIN_CELESTE_THRESHOLD);
+
+                // Mask resolution - 1 sets it to fine
+                bool t = (wxConfigBase::Get()->Read(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER) != 0);
+                if (t){
+                    //cerr <<"---Celeste--- Using small filter" << endl;
+                    gRadius = 10;
+                    spacing = (gRadius * 2) + 1;
+                }
+
+                // Image to analyse
+                string imagefile = m_pano->getImage(imgNr).getFilename();
+
+                // Print progress
+                MainFrame::Get()->SetStatusText(_("searching for cloud-like control points..."),0);
+
+                // Vector to store Gabor filter responses
+                vector<double> svm_responses_ap;
+                string mask_format = "PNG";
+                unsigned int mask = 0;
+
+                // Get responses
+                get_gabor_response(imagefile, mask, strModelFileName, threshold, mask_format, svm_responses_ap);
+
+                // Print SVM results
+                for (unsigned int c = 0; c < svm_responses_ap.size(); c++){
+
+                    unsigned int pNr = global_cp_nr[c] - removed;	
+
+                    if (svm_responses_ap[c] >= threshold){
+
+                        DEBUG_DEBUG("about to delete point " << pNr);
+                        GlobalCmdHist::getInstance().addCommand(
+                            new PT::RemoveCtrlPointCmd(*m_pano,pNr)
+                            );
+                        removed++;					
+                        cout << "CP: " << c << "\tSVM Score: " << svm_responses_ap[c] << "\tremoved." << endl;
+                    }
+                }
+                if (removed) cout << endl;
+
+            }
+        }else{	
+            wxMessageBox(_("Celeste model file not found, Hugin needs to be properly installed." ), _("Fatal Error"));
+        }	
+
+        MainFrame::Get()->SetStatusText(_(""),0);
+
+        // reset locale
+        setlocale(LC_NUMERIC,old_locale);
+
+    }
+    DEBUG_TRACE("Finished running Celeste");
+
+    progress.increaseProgress(1.0, std::wstring(wxString(_("Determining placement of the images")).wc_str(wxConvLocal)));
 
     // find components..
     CPGraph graph;
@@ -411,9 +538,9 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
         unsigned i2 = *(comps[1].begin());
         MainFrame::Get()->ShowCtrlPointEditor( i1, i2);
         // display message box with 
-        
+
         wxMessageBox(wxString::Format(_("Warning %d unconnected image groups found:"), n) + Components2Str(comps) + wxT("\n")
-                     + _("Please create control points between unconnected images using the Control Points tab.\n\nAfter adding the points, press the \"Align\" button again"),_("Error"), wxOK , this);
+            + _("Please create control points between unconnected images using the Control Points tab.\n\nAfter adding the points, press the \"Align\" button again"),_("Error"), wxOK , this);
         return;
     }
 
@@ -448,7 +575,7 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
         registerPTWXDlgFcn(MainFrame::Get());
     }
 
-    progress.increaseProgress(1.0, std::string(wxString(_("Leveling the panorama")).mb_str(wxConvLocal)));
+    progress.increaseProgress(1.0, std::wstring(wxString(_("Leveling the panorama")).wc_str(wxConvLocal)));
 
     // straighten
     optPano.straighten();
@@ -483,12 +610,12 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
     opts.setWidth(roundi(w*sizeFactor), true);
     optPano.setOptions(opts);
 
-    progress.increaseProgress(1.0, std::string(wxString(_("Loading images")).mb_str(wxConvLocal)));
+    progress.increaseProgress(1.0, std::wstring(wxString(_("Loading images")).wc_str(wxConvLocal)));
 
     // TODO: photometric optimisation.
     // first, ensure that vignetting and response coefficients are linked
     const char * varnames[] = {"Va", "Vb", "Vc", "Vd", "Vx", "Vy",
-                            "Ra", "Rb", "Rc", "Rd", "Re",  0};
+        "Ra", "Rb", "Rc", "Rd", "Re",  0};
 
     for (size_t i = 0; i < optPano.getNrOfLenses(); i++) {
         const Lens & l = optPano.getLens(i);
@@ -545,17 +672,17 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
         vigra::FRGBImage * img = new FRGBImage;
         if (!e) {
             wxMessageBox(_("Error: could not load all images"), 
-                    _("Error"),wxOK,this);
+                _("Error"),wxOK,this);
             return;
         }
         if (e->origType == "UINT8") {
             reduceToNextLevel(*(e->image8), *img);
             transformImage(vigra::srcImageRange(*img), vigra::destImage(*img),
-                            vigra::functor::Arg1()/vigra::functor::Param(255.0));
+                vigra::functor::Arg1()/vigra::functor::Param(255.0));
         } else if (e->origType == "UINT16") {
             reduceToNextLevel(*(e->image16), *img);
             transformImage(vigra::srcImageRange(*img), vigra::destImage(*img),
-                            vigra::functor::Arg1()/vigra::functor::Param(65535.0));
+                vigra::functor::Arg1()/vigra::functor::Param(65535.0));
         } else {
             reduceToNextLevel(*(e->imageFloat), *img);
         }
@@ -565,7 +692,7 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
     std::vector<vigra_ext::PointPairRGB> points;
     extractPoints(optPano, srcImgs, nPoints, randomPoints, *(MainFrame::Get()), points);
 
-    progress.increaseProgress(1.0, std::string(wxString(_("Vignetting and exposure correction")).mb_str(wxConvLocal)));
+    progress.increaseProgress(1.0, std::wstring(wxString(_("Vignetting and exposure correction")).wc_str(wxConvLocal)));
 
     PhotometricOptimizeMode poptmode = OPT_PHOTOMETRIC_LDR;
     if (opts.outputMode == PanoramaOptions::OUTPUT_HDR) {
@@ -573,7 +700,7 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
     }
     double error;
     smartOptimizePhotometric(optPano, poptmode,
-                                points, *(MainFrame::Get()), error);
+        points, *(MainFrame::Get()), error);
     cout << "Auto align, photometric error: " << error *255 << " grey values" << std::endl;
 
     // calculate the mean exposure.
@@ -594,7 +721,7 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
 
     // show preview frame
     wxCommandEvent dummy;
-    MainFrame::Get()->OnTogglePreviewFrame(dummy);
+    MainFrame::Get()->OnToggleGLPreviewFrame(dummy);
 
     // enable stitch button
     m_createButton->Enable();
@@ -662,7 +789,7 @@ void AssistantPanel::OnExifToggle (wxCommandEvent & e)
         double cropFactor = 0;
         double focalLength = 0;
         SrcPanoImage srcImg = m_pano->getSrcImage(imgNr);
-        bool ok = initImageFromFile(srcImg, focalLength, cropFactor);
+        bool ok = initImageFromFile(srcImg, focalLength, cropFactor, true);
         if (! ok) {
             if (!getLensDataFromUser(this, srcImg, focalLength, cropFactor)) {
                 // hmm, we don't know anything, assume a standart lens.
