@@ -9,7 +9,7 @@
  *
  *  Rewritten by Pablo d'Angelo
  *
- *  $Id: LensPanel.cpp 3346 2008-08-21 04:52:08Z ippei $
+ *  $Id: LensPanel.cpp 3955 2009-06-23 05:34:34Z tmodes $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -34,8 +34,6 @@
 
 #include <algorithm>
 
-#include "jhead/jhead.h"
-
 #include "common/wxPlatform.h"
 #include "hugin/LensPanel.h"
 #include "hugin/CommandHistory.h"
@@ -49,6 +47,7 @@
 #include "hugin/TextKillFocusHandler.h"
 #include "hugin/wxPanoCommand.h"
 //#include "hugin/VigCorrDialog.h"
+#include "hugin/ResetDialog.h"
 
 
 using namespace PT;
@@ -104,6 +103,7 @@ BEGIN_EVENT_TABLE(LensPanel, wxPanel) //wxEvtHandler)
     EVT_BUTTON ( XRCID("lens_button_load"), LensPanel::OnLoadLensParameters )
     EVT_BUTTON ( XRCID("lens_button_newlens"), LensPanel::OnNewLens )
     EVT_BUTTON ( XRCID("lens_button_changelens"), LensPanel::OnChangeLens )
+    EVT_BUTTON ( XRCID("lens_button_reset"), LensPanel::OnReset )
     EVT_CHECKBOX ( XRCID("lens_inherit_v"), LensPanel::OnVarInheritChanged )
     EVT_CHECKBOX ( XRCID("lens_inherit_a"), LensPanel::OnVarInheritChanged )
     EVT_CHECKBOX ( XRCID("lens_inherit_b"), LensPanel::OnVarInheritChanged )
@@ -353,6 +353,10 @@ void LensPanel::panoramaImagesChanged (PT::Panorama &pano, const PT::UIntSet & i
     } else if (m_selectedImages.size() == 0) {
         UpdateLensDisplay();
     }
+    if(pano.getNrOfImages()>0)
+        XRCCTRL(*this, "lens_button_reset", wxButton)->Enable();
+    else
+        XRCCTRL(*this, "lens_button_reset", wxButton)->Disable();
 }
 
 
@@ -803,29 +807,17 @@ void LensPanel::OnReadExif(wxCommandEvent & e)
             unsigned int imgNr = *it;
             // check file extension
             wxFileName file(wxString(pano->getImage(imgNr).getFilename().c_str(), HUGIN_CONV_FILENAME));
-#ifndef HUGIN_USE_EXIV2
-            if (file.GetExt().CmpNoCase(wxT("jpg")) == 0 ||
-                file.GetExt().CmpNoCase(wxT("jpeg")) == 0 )
-            {
-#endif
-                double cropFactor = 0;
-                double focalLength = 0;
-                SrcPanoImage srcImg = pano->getSrcImage(imgNr);
-                bool ok = initImageFromFile(srcImg, focalLength, cropFactor);
-                if (! ok) {
-                    if ( ! getLensDataFromUser(this, srcImg, focalLength, cropFactor)) {
-                        srcImg.setHFOV(50);
-                    }
+            double cropFactor = 0;
+            double focalLength = 0;
+            SrcPanoImage srcImg = pano->getSrcImage(imgNr);
+            bool ok = initImageFromFile(srcImg, focalLength, cropFactor, true);
+            if (! ok) {
+                if ( ! getLensDataFromUser(this, srcImg, focalLength, cropFactor)) {
+                    srcImg.setHFOV(50);
                 }
-                //initLensFromFile(pano->getImage(imgNr).getFilename().c_str(), c, lens, vars, imgopts, true);
-                GlobalCmdHist::getInstance().addCommand(
-                        new PT::UpdateSrcImageCmd( *pano, imgNr, srcImg)
-                    );
-#ifndef HUGIN_USE_EXIV2
-            } else {
-                wxLogError(_("Not a jpeg file:") + file.GetName());
             }
-#endif
+            //initLensFromFile(pano->getImage(imgNr).getFilename().c_str(), c, lens, vars, imgopts, true);
+            GlobalCmdHist::getInstance().addCommand(new PT::UpdateSrcImageCmd( *pano, imgNr, srcImg));
         }
     } else {
         wxLogError(_("Please select an image and try again"));
@@ -896,39 +888,21 @@ void LensPanel::OnSaveLensParameters(wxCommandEvent & e)
                 cfg.Write(wxT("Lens/crop/right"), imgopts.cropRect.right());
                 cfg.Write(wxT("Lens/crop/bottom"), imgopts.cropRect.bottom());
 
-                // try to read the exif data and add that to the lens ini file
-                wxFileName file(wxString(pano->getImage(imgNr).getFilename().c_str(), HUGIN_CONV_FILENAME));
-                if (file.GetExt().CmpNoCase(wxT("jpg")) == 0 ||
-                        file.GetExt().CmpNoCase(wxT("jpeg")) == 0 )
-                {
-
-                    // TODO: use EXIF2 for reading this exif data.
-                    ImageInfo_t exif;
-                    ResetJpgfile();
-                    // Start with an empty image information structure.
-
-                    memset(&exif, 0, sizeof(exif));
-                    exif.FlashUsed = -1;
-                    exif.MeteringMode = -1;
-
-                    if (ReadJpegFile(exif,pano->getImage(imgNr).getFilename().c_str(), READ_EXIF)){
-                        // calculate crop factor, if possible
-                        double cropFactor=0;
-                        if (exif.FocalLength > 0 && exif.CCDHeight > 0 && exif.CCDWidth > 0) {
-                            cropFactor = sqrt(36.0*36.0+24.0*24)/sqrt(exif.CCDWidth*exif.CCDWidth + exif.CCDHeight*exif.CCDHeight);
-                        } else if (exif.FocalLength35mm > 0 && exif.FocalLength > 0) {
-                            cropFactor = exif.FocalLength35mm / exif.FocalLength;
-                        }
-
-                        // write exif data to ini file
-                        cfg.Write(wxT("EXIF/CameraMake"),  wxString(exif.CameraMake, wxConvLocal));
-                        cfg.Write(wxT("EXIF/CameraModel"), wxString(exif.CameraModel, wxConvLocal));
-                        cfg.Write(wxT("EXIF/FocalLength"), (double) exif.FocalLength);
-                        cfg.Write(wxT("EXIF/Aperture"),    (double) exif.ApertureFNumber);
-                        cfg.Write(wxT("EXIF/ISO"),         exif.ISOequivalent);
-                        cfg.Write(wxT("EXIF/CropFactor"),  cropFactor); 
-                        cfg.Write(wxT("EXIF/Distance"),    exif.Distance); 
-                    }
+                SrcPanoImage image = pano->getSrcImage(imgNr);
+                // No EXIF data is stored in the pano model so they will have
+                // to be loaded now to save into the lens file
+                double focalLength = 0;
+                double cropFactor = 0;
+                if (image.readEXIF(focalLength, cropFactor, false, false)) {
+                
+                    // write exif data to ini file
+                    cfg.Write(wxT("EXIF/CameraMake"),  wxString(image.getExifMake().c_str(), wxConvLocal));
+                    cfg.Write(wxT("EXIF/CameraModel"), wxString(image.getExifModel().c_str(), wxConvLocal));
+                    cfg.Write(wxT("EXIF/FocalLength"), image.getExifFocalLength());
+                    cfg.Write(wxT("EXIF/Aperture"), image.getExifAperture());
+                    cfg.Write(wxT("EXIF/ISO"), image.getExifISO());
+                    cfg.Write(wxT("EXIF/CropFactor"), image.getExifCropFactor()); 
+                    cfg.Write(wxT("EXIF/Distance"), image.getExifDistance()); 
                 }
 
                 cfg.Flush();
@@ -1112,6 +1086,110 @@ void LensPanel::OnChangeLens(wxCommandEvent & e)
     }
 }
 
+void LensPanel::OnReset(wxCommandEvent & e)
+{
+  ResetDialog reset_dlg(this);
+  if(reset_dlg.ShowModal()==wxID_OK)
+  {
+     //reset 
+    UIntSet selImg = images_list->GetSelected();
+    if ( selImg.size() < 1) {
+        // add all images.
+        selImg.clear();
+        unsigned int nImg = pano->getNrOfImages();
+        for (unsigned int i=0; i < nImg; i++) {
+            selImg.insert(i);
+        }
+    }
+
+    if (selImg.size() == 0) {
+        return;
+    }
+    VariableMapVector vars;
+    for(UIntSet::const_iterator it = selImg.begin(); it != selImg.end(); it++)
+    {
+        unsigned int imgNr = *it;
+        VariableMap ImgVars=pano->getImageVariables(imgNr);
+        if(reset_dlg.GetResetPos())
+        {
+            map_get(ImgVars,"y").setValue(0);
+            map_get(ImgVars,"p").setValue(0);
+            map_get(ImgVars,"r").setValue(0);
+        };
+        double cropFactor = 0;
+        double focalLength = 0;
+        double eV = 0;
+        SrcPanoImage srcImg = pano->getSrcImage(imgNr);
+        if(reset_dlg.GetResetFOV() || reset_dlg.GetResetExposure())
+            srcImg.readEXIF(focalLength,cropFactor,eV,false,false);
+        if(reset_dlg.GetResetFOV())
+        {
+            if(focalLength!=0&&cropFactor!=0)
+            {
+                double newHFOV=calcHFOV(srcImg.getProjection(),focalLength,cropFactor,srcImg.getSize());
+                if(newHFOV!=0)
+                    map_get(ImgVars,"v").setValue(newHFOV);
+            };
+        };
+        if(reset_dlg.GetResetLens())
+        {
+            map_get(ImgVars,"a").setValue(0);
+            map_get(ImgVars,"b").setValue(0);
+            map_get(ImgVars,"c").setValue(0);
+            map_get(ImgVars,"d").setValue(0);
+            map_get(ImgVars,"e").setValue(0);
+            map_get(ImgVars,"g").setValue(0);
+            map_get(ImgVars,"t").setValue(0);
+        };
+        if(reset_dlg.GetResetExposure())
+        {
+            if(reset_dlg.GetResetExposureToExif())
+            {
+                //reset to exif value
+                if(eV!=0)
+                    map_get(ImgVars,"Eev").setValue(eV);
+            }
+            else
+            {
+                //reset to zero
+                map_get(ImgVars,"Eev").setValue(0);
+            };
+        };
+        if(reset_dlg.GetResetColor())
+        {
+            map_get(ImgVars,"Er").setValue(1);
+            map_get(ImgVars,"Eb").setValue(1);
+        };
+        if(reset_dlg.GetResetVignetting())
+        {
+            map_get(ImgVars,"Vb").setValue(0);
+            map_get(ImgVars,"Vc").setValue(0);
+            map_get(ImgVars,"Vd").setValue(0);
+            map_get(ImgVars,"Vx").setValue(0);
+            map_get(ImgVars,"Vy").setValue(0);
+
+        };
+        if(reset_dlg.GetResetResponse())
+        {
+            map_get(ImgVars,"Ra").setValue(0);
+            map_get(ImgVars,"Rb").setValue(0);
+            map_get(ImgVars,"Rc").setValue(0);
+            map_get(ImgVars,"Rd").setValue(0);
+            map_get(ImgVars,"Re").setValue(0);
+        };
+        vars.push_back(ImgVars);    
+    };
+    GlobalCmdHist::getInstance().addCommand(new PT::UpdateImagesVariablesCmd(*pano, selImg,vars));
+    if(reset_dlg.GetResetExposure())
+    {
+        //reset panorama output exposure value
+        PanoramaOptions opt = pano->getOptions();
+        opt.outputExposureValue = calcMeanExposure(*pano);
+        GlobalCmdHist::getInstance().addCommand(new PT::SetPanoOptionsCmd(*pano,opt));
+    };
+  }
+};
+
 const char *LensPanel::m_varNames[] = { "v", "a", "b", "c", "d", "e", "g", "t",
                                   "Eev", "Er", "Eb", 
                                   "Vb", "Vc", "Vd", "Vx", "Vy",
