@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: MainFrame.cpp 3373 2008-08-29 04:33:12Z ippei $
+ *  $Id: MainFrame.cpp 3891 2009-05-29 05:23:39Z tmodes $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -26,6 +26,8 @@
 
 #include <config.h>
 #include <hugin_version.h>
+#include <exiv2/exif.hpp>
+#include <exiv2/image.hpp>
 
 #include "panoinc_WX.h"
 #include "panoinc.h"
@@ -34,8 +36,6 @@
 
 #include "vigra_ext/Correlation.h"
 
-#include "jhead/jhead.h"
-
 #include "PT/utils.h"
 
 #include "hugin/config_defaults.h"
@@ -51,6 +51,7 @@
 #include "hugin/OptimizePanel.h"
 #include "hugin/OptimizePhotometricPanel.h"
 #include "hugin/PreviewFrame.h"
+#include "hugin/GLPreviewFrame.h"
 #include "hugin/huginApp.h"
 #include "hugin/CPEditorPanel.h"
 #include "hugin/CPListFrame.h"
@@ -139,6 +140,8 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("action_save_project"),  MainFrame::OnSaveProject)
     EVT_MENU(XRCID("action_save_as_project"),  MainFrame::OnSaveProjectAs)
     EVT_MENU(XRCID("action_save_as_ptstitcher"),  MainFrame::OnSavePTStitcherAs)
+	EVT_MENU(XRCID("action_send_to_batch"),  MainFrame::OnSendToBatch)
+	EVT_MENU(XRCID("action_open_batch_processor"),  MainFrame::OnOpenPTBatcher)
     EVT_MENU(XRCID("action_apply_template"),  MainFrame::OnApplyTemplate)
     EVT_MENU(XRCID("action_exit_hugin"),  MainFrame::OnUserQuit)
     EVT_MENU(XRCID("action_show_about"),  MainFrame::OnAbout)
@@ -151,7 +154,9 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("ID_EDITUNDO"), MainFrame::OnUndo)
     EVT_MENU(XRCID("ID_EDITREDO"), MainFrame::OnRedo)
     EVT_MENU(XRCID("ID_SHOW_PREVIEW_FRAME"), MainFrame::OnTogglePreviewFrame)
+    EVT_MENU(XRCID("ID_SHOW_GL_PREVIEW_FRAME"), MainFrame::OnToggleGLPreviewFrame)
     EVT_BUTTON(XRCID("ID_SHOW_PREVIEW_FRAME"),MainFrame::OnTogglePreviewFrame)
+    EVT_BUTTON(XRCID("ID_SHOW_GL_PREVIEW_FRAME"), MainFrame::OnToggleGLPreviewFrame)
 
     EVT_MENU(XRCID("action_optimize"),  MainFrame::OnOptimize)
     EVT_BUTTON(XRCID("action_optimize"),  MainFrame::OnOptimize)
@@ -310,6 +315,7 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
     opt_photo_panel->Init(&pano);
 
     preview_frame = new PreviewFrame(this, pano);
+    gl_preview_frame = new GLPreviewFrame(this, pano);
 
     // set the minimize icon
 #ifdef __WXMSW__
@@ -373,6 +379,7 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
 
     // set progress display for image cache.
     ImageCache::getInstance().setProgressDisplay(this);
+    ImageCache::getInstance().SetUpperLimit(wxConfigBase::Get()->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND));
 
     if(splash) {
         splash->Close();
@@ -541,6 +548,7 @@ void MainFrame::OnSaveProject(wxCommandEvent & e)
             wxString resultFnwx = scriptName.GetFullPath();
             resultFn = resultFnwx.mb_str(HUGIN_CONV_FILENAME);
             resultFn = utils::stripPath(utils::stripExtension(resultFn));
+            std::string tmpDir((wxConfigBase::Get()->Read(wxT("tempDir"),wxT(""))).mb_str(HUGIN_CONV_FILENAME));
 
             std::vector<std::string> outputFiles;
             HuginBase::PanoramaMakefileExport::createMakefile(pano,
@@ -550,7 +558,8 @@ void MainFrame::OnSaveProject(wxCommandEvent & e)
                                                               progs,
                                                               "",
                                                               outputFiles,
-                                                              makefile);
+                                                              makefile,
+                                                              tmpDir);
         }
     }
     SetStatusText(wxString::Format(_("saved project %s"), m_filename.c_str()),0);
@@ -767,8 +776,16 @@ void MainFrame::OnAddImages( wxCommandEvent& event )
       dlg.SetFilterIndex(0);
     else if (img_ext == wxT("jpg"))
       dlg.SetFilterIndex(1);
-    else if (img_ext == wxT("all files"))
+    else if (img_ext == wxT("tiff"))
       dlg.SetFilterIndex(2);
+    else if (img_ext == wxT("png"))
+      dlg.SetFilterIndex(3);
+    else if (img_ext == wxT("hdr"))
+      dlg.SetFilterIndex(4);
+    else if (img_ext == wxT("exr"))
+      dlg.SetFilterIndex(5);
+    else if (img_ext == wxT("all files"))
+      dlg.SetFilterIndex(6);
     DEBUG_INFO ( "Image extention: " << img_ext.mb_str(wxConvLocal) )
 
     // call the file dialog
@@ -801,7 +818,11 @@ void MainFrame::OnAddImages( wxCommandEvent& event )
         switch ( dlg.GetFilterIndex() ) {
         case 0: config->Write(wxT("lastImageType"), wxT("all images")); break;
         case 1: config->Write(wxT("lastImageType"), wxT("jpg")); break;
-        case 2: config->Write(wxT("lastImageType"), wxT("all files")); break;
+        case 2: config->Write(wxT("lastImageType"), wxT("tiff")); break;
+        case 3: config->Write(wxT("lastImageType"), wxT("png")); break;
+        case 4: config->Write(wxT("lastImageType"), wxT("hdr")); break;
+        case 5: config->Write(wxT("lastImageType"), wxT("exr")); break;
+        case 6: config->Write(wxT("lastImageType"), wxT("all files")); break;
         }
 
     } else {
@@ -812,22 +833,51 @@ void MainFrame::OnAddImages( wxCommandEvent& event )
     DEBUG_TRACE("");
 }
 
-time_t ReadJpegTime(const char* filename)
+time_t ReadExifTime(const char* filename)
 {
-    // Skip it if it isn't JPEG or there's no EXIF.
-    ResetJpgfile();
-    ImageInfo_t exif;
-    memset(&exif, 0, sizeof(exif));
-    if (!ReadJpegFile(exif, filename, READ_EXIF))
+    Exiv2::Image::AutoPtr image;
+    try {
+        image = Exiv2::ImageFactory::open(filename);
+    }catch(...) {
         return 0;
+    }
+    if (image.get() == 0) {
+        return 0;
+    }
+
+    image->readMetadata();
+    Exiv2::ExifData &exifData = image->exifData();
+    if (exifData.empty()) {
+        return 0;
+    }
+
+    Exiv2::Exifdatum& tag = exifData["Exif.Image.DateTime"];
+    const std::string date_time = tag.toString();
+
     // Remember the file and a shutter timestamp.
-    time_t stamp;
     struct tm when;
     memset(&when, 0, sizeof(when));
-    Exif2tm(&when, exif.DateTime);
+    when.tm_wday = -1;
+
+    // parse into the tm_structure
+    const int a = sscanf(date_time.c_str(), "%d:%d:%d %d:%d:%d",
+            &when.tm_year, &when.tm_mon, &when.tm_mday,
+            &when.tm_hour, &when.tm_min, &when.tm_sec);
+
+    if (a == 6){
+        when.tm_isdst = -1;
+        when.tm_mon -= 1;      // Adjust for unix zero-based months
+        when.tm_year -= 1900;  // Adjust for year starting at 1900
+    } else {
+        // Not in EXIF format
+        return 0;
+    }
+
+    time_t stamp;
     stamp = mktime(&when);
     if (stamp == (time_t)(-1))
         return 0;
+
     return stamp;
 }
 
@@ -903,7 +953,7 @@ void MainFrame::OnAddTimeImages( wxCommandEvent& event )
     {
         wxString file = found->first;
         // Check the time if it's got a camera EXIF timestamp.
-		  time_t stamp = ReadJpegTime(file.mb_str(HUGIN_CONV_FILENAME));
+		  time_t stamp = ReadExifTime(file.mb_str(HUGIN_CONV_FILENAME));
       	  if (stamp) {
             filenames[file] = stamp;
             timeMap[(const char *)file.mb_str(HUGIN_CONV_FILENAME)] = stamp;
@@ -1107,6 +1157,8 @@ void MainFrame::OnShowPrefs(wxCommandEvent & e)
 //    wxDialog dlg(this, -1, _("Preferences"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxDIALOG_MODAL);
     PreferencesDialog * pref_dlg = new PreferencesDialog(this);
     pref_dlg->ShowModal();
+    //update image cache size
+    ImageCache::getInstance().SetUpperLimit(wxConfigBase::Get()->Read(wxT("/ImageCache/UpperBound"), HUGIN_IMGCACHE_UPPERBOUND));
 
 }
 
@@ -1131,6 +1183,15 @@ void MainFrame::OnTogglePreviewFrame(wxCommandEvent & e)
 	preview_frame->OnUpdate(dummy);
 }
 
+void MainFrame::OnToggleGLPreviewFrame(wxCommandEvent & e)
+{
+    if (gl_preview_frame->IsIconized()) {
+        gl_preview_frame->Iconize(false);
+    }
+    gl_preview_frame->Show();
+    gl_preview_frame->Raise();
+}
+
 void MainFrame::OnShowCPFrame(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
@@ -1188,6 +1249,21 @@ void MainFrame::OnApplyTemplate(wxCommandEvent & e)
 }
 
 
+void MainFrame::OnSendToBatch(wxCommandEvent & e)
+{
+	pano_panel->SendToBatch();
+}
+
+void MainFrame::OnOpenPTBatcher(wxCommandEvent & e)
+{
+#ifdef __WINDOWS__
+	wxString huginPath = getExePath(wxGetApp().argv[0])+wxFileName::GetPathSeparator();
+#else
+	wxString huginPath = _T("");	//we call the batch processor directly without path on linux
+#endif	
+	wxExecute(huginPath+_T("PTBatcherGUI"));
+}
+
 void MainFrame::OnFineTuneAll(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
@@ -1222,9 +1298,7 @@ void MainFrame::OnFineTuneAll(wxCommandEvent & e)
                               HUGIN_FT_CURV_THRESHOLD);
 
     {
-    MyProgressDialog pdisp(_("Fine-tuning all points"), wxT(""), NULL, wxPD_ELAPSED_TIME | wxPD_AUTO_HIDE | wxPD_APP_MODAL );
-
-    pdisp.pushTask(ProgressTask((const char *)wxString(_("Finetuning")).mb_str(wxConvLocal),"",1.0/unoptimized.size()));
+    ProgressReporterDialog progress(unoptimized.size(),_("Fine-tuning all points"),_("Finetuning"),this);
 
     ImageCache & imgCache = ImageCache::getInstance();
 
@@ -1238,7 +1312,7 @@ void MainFrame::OnFineTuneAll(wxCommandEvent & e)
 
         while (it != unoptimized.end()) {
             if (cps[*it].image1Nr == imgNr || cps[*it].image2Nr == imgNr) {
-                pdisp.increase();
+                progress.increaseProgress(1);
                 if (cps[*it].mode == ControlPoint::X_Y) {
                     // finetune only normal points
                     DEBUG_DEBUG("fine tuning point: " << *it);
@@ -1398,6 +1472,7 @@ void MainFrame::enableTools(bool option)
 	wxToolBar* theToolBar = GetToolBar();
 	theToolBar->EnableTool(XRCID("action_optimize"), option);
 	theToolBar->EnableTool(XRCID("ID_SHOW_PREVIEW_FRAME"), option);
+	theToolBar->EnableTool(XRCID("ID_SHOW_GL_PREVIEW_FRAME"), option);
 	//theToolBar->EnableTool(XRCID("action_save_project"), option);
 	//theToolBar->EnableTool(XRCID("action_save_as_project"), option);
     }
@@ -1406,6 +1481,7 @@ void MainFrame::enableTools(bool option)
 	theMenuBar->Enable(XRCID("action_optimize"), option);
 	theMenuBar->Enable(XRCID("action_finetune_all_cp"), option);
 	theMenuBar->Enable(XRCID("ID_SHOW_PREVIEW_FRAME"), option);
+	theMenuBar->Enable(XRCID("ID_SHOW_GL_PREVIEW_FRAME"), option);
 	//theMenuBar->Enable(XRCID("action_save_project"), option);
 	//theMenuBar->Enable(XRCID("action_save_as_project"), option);
 	//theMenuBar->Enable(XRCID("action_save_as_ptstitcher"), option);
