@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: OptimizePanel.cpp 3356 2008-08-23 22:24:30Z gpatters1 $
+ *  $Id: OptimizePanel.cpp 3721 2009-03-11 21:48:19Z gkohlmeyer $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -100,7 +100,7 @@ bool OptimizePanel::Create(wxWindow* parent, wxWindowID id , const wxPoint& pos,
 
     m_only_active_images_cb = XRCCTRL(*this, "optimizer_only_active_images", wxCheckBox);
     DEBUG_ASSERT(m_only_active_images_cb);
-    m_only_active_images_cb->SetValue(wxConfigBase::Get()->Read(wxT("/OptimizePanel/OnlyActiveImages"),1l));
+    m_only_active_images_cb->SetValue(wxConfigBase::Get()->Read(wxT("/OptimizePanel/OnlyActiveImages"),1l) != 0);
 
 
     m_yaw_list = XRCCTRL(*this, "optimizer_yaw_list", wxCheckListBox);
