@@ -7,7 +7,7 @@
  *  @author Kai-Uwe Behrmann <web@tiscali.de> and
  *          Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoPanel.cpp 3381 2008-08-30 09:50:38Z ippei $
+ *  $Id: PanoPanel.cpp 3809 2009-04-28 05:33:07Z tmodes $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -26,6 +26,7 @@
  */
 
 #include <config.h>
+#include <wx/stdpaths.h>
 
 #include "panoinc_WX.h"
 #include "panoinc.h"
@@ -53,6 +54,7 @@ extern "C" {
 //#include "hugin/LensPanel.h"
 //#include "hugin/ImagesPanel.h"
 #include "hugin/CPImageCtrl.h"
+#include "hugin/CPImagesComboBox.h"
 #include "hugin/PanoPanel.h"
 #include "hugin/MainFrame.h"
 #include "hugin/huginApp.h"
@@ -83,6 +85,7 @@ BEGIN_EVENT_TABLE(PanoPanel, wxPanel)
     EVT_TEXT_ENTER ( XRCID("pano_val_roi_right"),PanoPanel::ROIChanged )
     EVT_BUTTON ( XRCID("pano_button_opt_width"), PanoPanel::DoCalcOptimalWidth)
     EVT_BUTTON ( XRCID("pano_button_stitch"),PanoPanel::OnDoStitch )
+	EVT_BUTTON ( XRCID("pano_button_batch"),PanoPanel::OnSendToBatch )
 
     EVT_CHECKBOX ( XRCID("pano_cb_ldr_output_blended"), PanoPanel::OnOutputFilesChanged)
     EVT_CHECKBOX ( XRCID("pano_cb_ldr_output_layers"), PanoPanel::OnOutputFilesChanged)
@@ -142,6 +145,26 @@ bool PanoPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, cons
     m_keepViewOnResize = true;
     m_hasStacks=false;
 
+#ifdef ThisNeverHappens
+// provide some translatable strings for the drop down menu
+    wxLogMessage(_("Fisheye"));
+    wxLogMessage(_("Stereographic"));
+    wxLogMessage(_("Mercator"));
+    wxLogMessage(_("Trans Mercator"));
+    wxLogMessage(_("Sinusoidal"));
+    wxLogMessage(_("Lambert Cylindrical Equal Area"));
+    wxLogMessage(_("Lambert Equal Area Azimuthal"));
+    wxLogMessage(_("Albers Equal Area Conic"));
+    wxLogMessage(_("Miller Cylindrical"));
+    wxLogMessage(_("Panini"));
+    wxLogMessage(_("Architectural"));
+    wxLogMessage(_("Orthographic"));
+    wxLogMessage(_("Equisolid"));
+    wxLogMessage(_("Equirectangular Panini"));
+    wxLogMessage(_("Biplane"));
+    wxLogMessage(_("Triplane"));
+#endif
+
     /* populate with all available projection types */
 #ifdef HasPANO13
     int nP = panoProjectionFormatCount();
@@ -216,6 +239,8 @@ bool PanoPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, cons
 
     m_StitchButton = XRCCTRL(*this, "pano_button_stitch", wxButton);
     DEBUG_ASSERT(m_StitchButton);
+	m_BatchButton = XRCCTRL(*this, "pano_button_batch", wxButton);
+    DEBUG_ASSERT(m_BatchButton);
 
     m_FileFormatChoice = XRCCTRL(*this, "pano_choice_file_format", wxChoice);
     DEBUG_ASSERT(m_FileFormatChoice);
@@ -285,6 +310,9 @@ PanoPanel::~PanoPanel(void)
 void PanoPanel::panoramaChanged (PT::Panorama &pano)
 {
     DEBUG_TRACE("");
+	bool hasImages = pano.getActiveImages().size() > 0;
+    m_StitchButton->Enable(hasImages);
+	m_BatchButton->Enable(hasImages);
 
 #ifdef STACK_CHECK //Disabled for 0.7.0 release
     const bool hasStacks = StackCheck(pano);
@@ -987,11 +1015,46 @@ void PanoPanel::DoStitch()
 #endif
 }
 
+void PanoPanel::SendToBatch()
+{
+	wxCommandEvent dummy;
+	OnSendToBatch(dummy);
+}
+
 void PanoPanel::OnDoStitch ( wxCommandEvent & e )
 {
     DoStitch();
 }
 
+void PanoPanel::OnSendToBatch ( wxCommandEvent & e )
+{
+	wxCommandEvent dummy;
+	MainFrame::Get()->OnSaveProject(dummy);
+	wxString projectFile = MainFrame::Get()->getProjectName();
+	if(wxFileName::FileExists(projectFile))
+	{
+#ifdef __WINDOWS__
+		wxString huginPath = getExePath(wxGetApp().argv[0])+wxFileName::GetPathSeparator();
+#else
+		wxString huginPath = _T("");	//we call the batch processor directly without path on linux
+#endif	
+		wxExecute(huginPath+wxT("PTBatcherGUI ")+wxQuoteFilename(projectFile));
+
+		/*int i=0;
+		wxString batchFileName = wxStandardPaths::Get().GetUserConfigDir()+wxFileName::GetPathSeparator();
+		batchFileName = batchFileName.Append(_T(".ptbs")) << i;
+		while(wxFileName::FileExists(batchFileName)){
+			i++;
+			batchFileName = wxStandardPaths::Get().GetUserConfigDir()+wxFileName::GetPathSeparator();
+			batchFileName = batchFileName.Append(_T(".ptbs")) << i;
+		}
+		wxFile batchFile;
+		batchFile.Create(batchFileName);
+		batchFile.Write(projectFile,wxConvLocal);
+		batchFile.Close(); */
+	}
+}
+
 void PanoPanel::FileFormatChanged(wxCommandEvent & e)
 {
 
