@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PreferencesDialog.cpp 3212 2008-07-20 02:10:45Z gpatters1 $
+ *  $Id: PreferencesDialog.cpp 3870 2009-05-21 20:46:21Z brunopostle $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -102,9 +102,9 @@ PreferencesDialog::PreferencesDialog(wxWindow *parent)
     wxXmlResource::Get()->LoadDialog(this, parent, wxT("pref_dialog"));
 
 #ifdef __WXMSW__
-    wxIcon myIcon(MainFrame::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(MainFrame::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
 #endif
     SetIcon(myIcon);
 
@@ -124,12 +124,18 @@ PreferencesDialog::PreferencesDialog(wxWindow *parent)
     *lp = wxLANGUAGE_DEFAULT;
     lang_choice->Append(_("System default"), lp);
     lp = new long;
+    *lp = wxLANGUAGE_BULGARIAN;
+    lang_choice->Append(_("Bulgarian"), lp);
+    lp = new long;
     *lp = wxLANGUAGE_CATALAN;
     lang_choice->Append(_("Catalan"), lp);
     lp = new long;
     *lp = wxLANGUAGE_CHINESE_SIMPLIFIED;
     lang_choice->Append(_("Chinese (Simplified)"), lp);
     lp = new long;
+    *lp = wxLANGUAGE_CHINESE_TRADITIONAL;
+    lang_choice->Append(_("Chinese (Traditional)"), lp);
+    lp = new long;
     *lp = wxLANGUAGE_CZECH;
     lang_choice->Append(_("Czech"), lp);
     lp = new long;
@@ -154,6 +160,9 @@ PreferencesDialog::PreferencesDialog(wxWindow *parent)
     *lp = wxLANGUAGE_JAPANESE;
     lang_choice->Append(_("Japanese"), lp);
     lp = new long;
+    *lp = wxLANGUAGE_KOREAN;
+    lang_choice->Append(_("Korean"), lp);
+    lp = new long;
     *lp = wxLANGUAGE_POLISH;
     lang_choice->Append(_("Polish"), lp);
     lp = new long;
@@ -166,6 +175,9 @@ PreferencesDialog::PreferencesDialog(wxWindow *parent)
     *lp = wxLANGUAGE_SLOVAK;
     lang_choice->Append(_("Slovak"), lp);
     lp = new long;
+    *lp = wxLANGUAGE_SLOVENIAN;
+    lang_choice->Append(_("Slovenian"), lp);
+    lp = new long;
     *lp = wxLANGUAGE_SPANISH;
     lang_choice->Append(_("Spanish"), lp);
     lp = new long;
@@ -768,6 +780,24 @@ void PreferencesDialog::UpdateDisplayData(int panel)
         MY_STR_VAL("prefs_enblend_EnfuseArgs", cfg->Read(wxT("/Enfuse/Args"),
                                                       wxT(HUGIN_ENFUSE_ARGS)));
     }
+    
+    if (panel==0 || panel == 6) {
+        // Celeste settings
+
+        t = cfg->Read(wxT("/Celeste/Auto"), HUGIN_CELESTE_AUTO) == 1;
+        MY_BOOL_VAL("prefs_celeste_auto", t);
+	
+        d=HUGIN_CELESTE_THRESHOLD;
+        cfg->Read(wxT("/Celeste/Threshold"), &d, HUGIN_CELESTE_THRESHOLD);
+        tstr = utils::doubleTowxString(d);
+        MY_STR_VAL("prefs_celeste_threshold", tstr);
+
+	MY_CHOICE_VAL("prefs_celeste_filter", cfg->Read(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER));
+
+    }
+
+
+
 
 /*
     // Panotools settings
@@ -863,6 +893,14 @@ void PreferencesDialog::OnRestoreDefaults(wxCommandEvent & e)
             cfg->Write(wxT("/Enfuse/Custom"), HUGIN_ENFUSE_EXE_CUSTOM);
             cfg->Write(wxT("/Enfuse/Args"), wxT(HUGIN_ENFUSE_ARGS));
         }
+	
+        if (noteb->GetSelection() == 5) {
+            /// Celeste
+            cfg->Write(wxT("/Celeste/Auto"), HUGIN_CELESTE_AUTO);
+            cfg->Write(wxT("/Celeste/Threshold"), HUGIN_CELESTE_THRESHOLD);
+            cfg->Write(wxT("/Celeste/Filter"), HUGIN_CELESTE_FILTER);
+        }	
+	
 /*
         if (noteb->GetSelection() == 5) {
             cfg->Write(wxT("/PTmender/Exe"), wxT(HUGIN_PT_MENDER_EXE) );
@@ -965,6 +1003,14 @@ void PreferencesDialog::UpdateConfigData()
     cfg->Write(wxT("/Enfuse/Exe"), MY_G_STR_VAL("prefs_enblend_EnfuseExe"));
     cfg->Write(wxT("/Enfuse/Args"), MY_G_STR_VAL("prefs_enblend_EnfuseArgs"));
 
+    // Celeste
+    cfg->Write(wxT("/Celeste/Auto"), MY_G_BOOL_VAL("prefs_celeste_auto"));
+    t = MY_G_STR_VAL("prefs_celeste_threshold");
+    td = HUGIN_CELESTE_THRESHOLD;
+    utils::stringToDouble(std::string(t.mb_str(wxConvLocal)), td);
+    cfg->Write(wxT("/Celeste/Threshold"), td);
+    cfg->Write(wxT("/Celeste/Filter"), MY_G_CHOICE_VAL("prefs_celeste_filter"));
+
     cfg->Flush();
     UpdateDisplayData(0);
 }
