@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PreviewFrame.cpp 3414 2008-09-12 03:05:57Z ippei $
+ *  $Id: PreviewFrame.cpp 3704 2009-03-04 19:14:20Z yuv $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -362,9 +362,9 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
 
     // set the minimize icon
 #ifdef __WXMSW__
-    wxIcon myIcon(MainFrame::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.ico"),wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(MainFrame::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
+    wxIcon myIcon(huginApp::Get()->GetXRCPath() + wxT("data/icon.png"),wxBITMAP_TYPE_PNG);
 #endif
     SetIcon(myIcon);
 
@@ -826,6 +826,8 @@ void PreviewFrame::OnChangeFOV(wxScrollEvent & e)
             }
         }
         opt.setProjectionParameters(para);
+		opt.setHFOV(m_HFOVSlider->GetValue());
+		opt.setVFOV(m_VFOVSlider->GetValue());
 #endif
     }
 
