@@ -7,7 +7,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: autooptimiser.cpp 3266 2008-08-04 13:08:48Z dangelo $
+ *  $Id: autooptimiser.cpp 3944 2009-06-17 05:34:34Z tmodes $
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -139,7 +139,7 @@ int main(int argc, char *argv[])
     const char * scriptFile = argv[optind];
 
     Panorama pano;
-    if (scriptFile == "-") {
+    if (scriptFile[0] == '-') {
         DocumentData::ReadWriteError err = pano.readData(std::cin);
         if (err != DocumentData::SUCCESSFUL) {
             cerr << "error while reading script file from stdin." << endl;
@@ -176,7 +176,7 @@ int main(int argc, char *argv[])
             double focalLength = 0;
             double cropFactor = 0;
             cerr << "HFOV of image " << img.getFilename() << " invalid, trying to read EXIF tags" << endl;
-            bool ok = img.readEXIF(focalLength, cropFactor, true);
+            bool ok = img.readEXIF(focalLength, cropFactor, true, false);
             if (! ok) {
                 if (hfov) {
                     img.setHFOV(hfov);
